import logging
import os

_logger = logging.getLogger(__name__)


# noinspection PyUnresolvedReferences
class SettingsPseudoBorg:
    api_hostname = str(os.environ.get("API_HOSTNAME", ""))
    metrics_server = str(os.environ.get("METRICS_SERVER", "127.0.0.1"))
    metrics_prefix = str(os.environ.get("METRICS_PREFIX", ""))
    metrics_port = str(os.environ.get("METRICS_PORT", "2003"))
    metrics_type = str(os.environ.get("METRICS_TYPE", "CARBON"))
    metrics_username = str(os.environ.get("METRICS_USERNAME", "admin"))
    metrics_password = str(os.environ.get("METRICS_PASSWORD", "test"))
    query_worker_count = int(os.environ.get("QUERY_WORKER_COUNT", "1"))
    query_interval = int(os.environ.get("QUERY_INTERVAL", "60"))
    reporter_interval = int(os.environ.get("REPORTER_INTERVAL", "60"))
    execution_mode = str(os.environ.get("EXECUTION_MODE", "performance"))
    timeout_seconds = int(os.environ.get("TIMEOUT_SECONDS", "30"))
    admin_bucket = str(os.environ.get("ADMIN_S3_BUCKET", ""))
    results_delay = int(os.environ.get("RESULTS_DELAY_SECONDS", "5"))
    number_buckets = int(os.environ.get("NUMBER_BUCKETS", "10"))
    env_id = str(os.environ.get("ENV_ID", "prod"))
    # Only set through command line
    config_path = str()

    def __str__(self):
        settings = SettingsPseudoBorg()
        pw = settings.metrics_password
        return (
            f"\tAPI_HOSTNAME:\t{settings.api_hostname}\n"
            f"\tMETRICS_SERVER:\t{settings.metrics_server}\n"
            f"\tMETRICS_PREFIX:\t{settings.metrics_prefix}\n"
            f"\tMETRICS_PORT:\t{settings.metrics_port}\n"
            f"\tMETRICS_TYPE:\t{settings.metrics_type.lower()}\n"
            f"\tMETRICS_USERNAME:\t{settings.metrics_username}\n"
            f'\tMETRICS_PASSWORD:\t{pw[:2]}{"*" * (len(pw) - 4)}{pw[-2:]}\n'
            f"\tQUERY_WORKER_COUNT:\t{settings.query_worker_count}\n"
            f"\tQUERY_INTERVAL:\t{settings.query_interval}\n"
            f"\tREPORTER_INTERVAL:\t{settings.reporter_interval}\n"
            f"\tEXECUTION_MODE:\t{settings.execution_mode}\n"
            f"\tTIMEOUT_SECONDS:\t{settings.timeout_seconds}\n"
            f"\tADMIN_S3_BUCKET:\t{settings.admin_bucket}\n"
            f"\tRESULTS_DELAY_SECONDS:\t{settings.results_delay}\n"
            f"\tNUMBER_BUCKETS:\t{settings.number_buckets}"
            f"\tENV_ID:\t{settings.env_id}\n"
        )


def get_settings():
    return SettingsPseudoBorg()


class MisconfigurationError(BaseException):
    pass


def get_full_prefix(q_position: int) -> str:
    settings = get_settings()
    prefix = "query_tool.operation"
    if settings.execution_mode == "performance":
        prefix += f".{q_position}"
    elif settings.execution_mode == "availability_summary":
        prefix += ".summary"
    elif settings.execution_mode == "availability_details":
        prefix += ".details"
    return prefix


def validate_settings():
    settings = get_settings()
    ok = all(
        (
            settings.api_hostname,
            settings.metrics_server,
            settings.metrics_prefix,
            settings.metrics_port,
            settings.metrics_type.lower() in ["carbon", "console", "influx"],
            # only need username/password for influx
            settings.metrics_username or settings.metrics_type.lower() != "influx",
            settings.metrics_password or settings.metrics_type.lower() != "influx",
            settings.query_worker_count >= 1,
            settings.query_interval,
            settings.reporter_interval,
            settings.execution_mode in ["availability", "performance", "availability_summary", "availability_details"],
            settings.timeout_seconds,
            settings.admin_bucket,
            settings.results_delay,
        )
    )
    if not ok:
        settings = get_settings()
        pw = settings.metrics_password
        msg = (
            f"\nPlease make sure all those settings have a valid value:\n"
            f"\tAPI_HOSTNAME:\t{settings.api_hostname}\n"
            f"\tMETRICS_SERVER:\t{settings.metrics_server}\n"
            f"\tMETRICS_PREFIX:\t{settings.metrics_prefix}\n"
            f"\tMETRICS_PORT:\t{settings.metrics_port}\n"
            f"\tMETRICS_TYPE:\t{settings.metrics_type.lower()}\n"
            f"\tMETRICS_USERNAME:\t{settings.metrics_username}\n"
            f'\tMETRICS_PASSWORD:\t{pw[:2]}{"*" * (len(pw) - 4)}{pw[-2:]}\n'
            f"\tQUERY_WORKER_COUNT:\t{settings.query_worker_count}\n"
            f"\tQUERY_INTERVAL:\t{settings.query_interval}\n"
            f"\tREPORTER_INTERVAL:\t{settings.reporter_interval}\n"
            f"\tEXECUTION_MODE:\t{settings.execution_mode}\n"
            f"\tTIMEOUT_SECONDS:\t{settings.timeout_seconds}\n"
            f"\tADMIN_S3_BUCKET:\t{settings.admin_bucket}\n"
            f"\tRESULTS_DELAY_SECONDS:\t{settings.results_delay}\n"
            f"\tENV_ID:\t{settings.env_id}\n"
        )
        _logger.error(msg)
        raise MisconfigurationError(msg)
