import logging
from asyncio import Queue

from time import time

from query_tool.workers.abstract_worker import AbstractWorker

_logger = logging.getLogger(__name__)


class QueryWorker(AbstractWorker):
    def __init__(self, macro_op_queue: Queue):
        super().__init__(macro_op_queue)
        self.time_start = time()

    async def run(self):
        await super().run()
        _logger.info("Starting QueryWorker")
        while True:
            _logger.debug("Worker waiting for macro op")
            macro_op = await self.macro_op_queue.get()
            _logger.debug(f"Executing macro op {macro_op.operation.id} for org {macro_op.org_id}")
            await macro_op.execute()
            if self.test_mode and self.macro_op_queue.empty():
                break
        self.dispose()

    def dispose(self):
        super().dispose()
